@@ -9,8 +9,9 @@ use auto_diff::optim::{SGD};
 use csv;
 use std::collections::{BTreeSet};
 use rand::prelude::*;
+extern crate openblas_src;
 
-use tensorboard_rs::summary_writer::SummaryWriter;
+//use tensorboard_rs::summary_writer::SummaryWriter;
 
 fn main() {
         let mut reader = csv::ReaderBuilder::new()
@@ -18,7 +19,6 @@ fn main() {
         .from_path("examples/data/wdbc.data")
         .expect("Cannot read wdbc.data");
 
-    let mut size = 0;
     let mut id;
     let mut ill;
     let mut ids = BTreeSet::<usize>::new();
@@ -27,7 +27,7 @@ fn main() {
     for record in reader.records() {
         let line = record.expect("");
         id = line[0].trim().parse::<usize>().expect("");
-        ill = line[1].trim().parse::<String>().expect("");
+        //ill = line[1].trim().parse::<String>().expect("");
         //println!("{}, {}", id, ill);
 
         if !ids.contains(&id) {
@@ -132,7 +132,7 @@ fn main() {
         //writer.flush();
 
         let output1 = output.clone();
-        let err = output1.sigmoid().unwrap().sub(&test_label.clone()).unwrap().abs().unwrap().sum(None, false).unwrap();
+        let err = (output1.sigmoid().unwrap() - test_label.clone()).abs().unwrap().sum(None, false).unwrap();
         println!("err: {:?}", err);
     }
 
