@@ -36,17 +36,14 @@
 //!
 
 
-extern crate ndarray;
-extern crate ndarray_linalg;
-
 pub mod var;
 pub mod op;
 pub mod optim;
 pub mod err;
 
 pub use var::{Var};
+pub use err::AutoDiffError;
 
 pub(crate) mod compute_graph;
 pub(crate) mod collection;
 pub(crate) mod var_inner;
-
