@@ -55,6 +55,14 @@ new_binary_op!(Matmul, "matmul",
                })
 );
 
+new_binary_op!(Outer, "outer",
+               (|a:&[Tensor], b:&[Tensor]|
+                b[0].swap(&a[0].outer(&a[1], None))),
+               (|input: &[Tensor], output_grad: &[Tensor], input_grad: &[Tensor]| {
+                   unimplemented!();
+               })
+);
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -62,13 +70,13 @@ mod tests {
 
     #[test]
     fn matmul() {
-        let mut op = Matmul::new();
+        let mut op = Mul::new();
 
-        //for i in 0..10 {
-        //    let zero = Tensor::from_vec_f64(&vec![(i - 5) as f64], &vec![1]);
-        //    let zero2 = zero.clone();
-        //    let good_grad = _gradient_checker(&mut op, &[zero, zero2], None, None, None);
-        //    assert_eq!(good_grad, true);                        
-        //}
+        for i in 0..10 {
+            let zero = Tensor::from_vec_f64(&vec![(i - 5) as f64], &vec![1]);
+            let zero2 = zero.clone();
+            let good_grad = _gradient_checker(&mut op, &[zero, zero2], None, None, None);
+            assert_eq!(good_grad, true);                        
+        }
     }
 }
