@@ -2,6 +2,8 @@ use std::cell::RefCell;
 use std::rc::Rc;
 use std::fmt;
 use ::rand::prelude::StdRng;
+use std::ops::{Add, Div, Neg, Sub, Mul};
+use std::convert::TryFrom;
 
 use tensor_rs::tensor::{Tensor};
 use crate::op::{Op};
@@ -11,8 +13,11 @@ use crate::var_inner::VarInner;
 use crate::compute_graph::{Net};
 
 
+
 macro_rules! var_1_to_1 {
-    ($a:ident) => {
+    ($(#[$attr:meta])*
+     $a:ident) => {
+        $(#[$attr])*
         pub fn $a(&self) -> Result<Var, AutoDiffError> {
             Ok(Var {
                 var: Rc::new(RefCell::new(self.var.borrow().$a()?))})
@@ -21,16 +26,20 @@ macro_rules! var_1_to_1 {
 }
 
 macro_rules! var_2_to_1 {
-    ($a:ident) => {
+    ($(#[$attr:meta])*
+     $a:ident) => {
+        $(#[$attr])*
         pub fn $a(&self, other: &Var) -> Result<Var, AutoDiffError> {
             Ok(Var {
-                var: Rc::new(RefCell::new(self.var.borrow().$a(&mut other.var.borrow_mut())?))})
+                var: Rc::new(RefCell::new(self.var.borrow().$a(&other.var.clone())?))})
         }
     }
 }
 
 macro_rules! var_more_to_1_with_para {
-    ($a:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
+    ($(#[$attr:meta])*
+     $a:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
+        $(#[$attr])*
         pub fn $a(&self, other: &[Var], $( $arg_name : $ArgTy ),*) -> Result<Var, AutoDiffError> {
             let mut other_input = Vec::new();
             for i in other {
@@ -43,7 +52,9 @@ macro_rules! var_more_to_1_with_para {
 }
 
 macro_rules! var_1_to_1_with_para {
-    ($a:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
+    ($(#[$attr:meta])*
+     $a:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
+        $(#[$attr])*
         pub fn $a(&self, $( $arg_name : $ArgTy ),*) -> Result<Var, AutoDiffError> {
             Ok(Var {
                 var: Rc::new(RefCell::new(self.var.borrow().$a($( $arg_name ),*)?))})
@@ -51,8 +62,21 @@ macro_rules! var_1_to_1_with_para {
     }
 }
 
+macro_rules! var_2_to_1_with_para {
+    ($(#[$attr:meta])*
+     $a:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
+        $(#[$attr])*
+        pub fn $a(&self, other: &Var, $( $arg_name : $ArgTy ),*) -> Result<Var, AutoDiffError> {
+            Ok(Var {
+                var: Rc::new(RefCell::new(self.var.borrow().$a(&other.var.clone(), $( $arg_name ),*)?))})
+        }
+    }
+}
+
 macro_rules! delegate_new_op {
-    ($a:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
+    ($(#[$attr:meta])*
+     $a:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
+        $(#[$attr])*
         pub fn $a($( $arg_name : $ArgTy ),*) -> Var {
             Var {
                 var: Rc::new(RefCell::new(VarInner::$a($( $arg_name ),*)))
@@ -87,6 +111,14 @@ impl Var {
         }
     }
 
+    /// Where it needs a assign operator,
+    /// we should use this ref_copy.
+    /// If a hard copy is necessary, then call clone().
+    pub fn ref_copy(self: &Var) -> Var {
+        Var {
+            var: self.var.clone(),
+        }
+    }
 
     pub fn size(&self) -> Vec<usize> {
         self.var.borrow().size()
@@ -94,17 +126,17 @@ impl Var {
     pub fn numel(&self) -> usize {
         self.var.borrow().numel()
     }
-    pub fn get_f32(&self, o: &[usize]) -> f32 {
+    pub fn get_f32(&self, o: &[usize]) -> Result<f32, AutoDiffError> {
         self.var.borrow().get_f32(o)
     }
-    pub fn set_f32(&self, o: &[usize], v: f32) {
-        self.var.borrow_mut().set_f32(o, v);
+    pub fn set_f32(&self, o: &[usize], v: f32) -> Result<(), AutoDiffError> {
+        self.var.borrow_mut().set_f32(o, v)
     }
-    pub fn get_f64(&self, o: &[usize]) -> f64 {
+    pub fn get_f64(&self, o: &[usize]) -> Result<f64, AutoDiffError> {
         self.var.borrow().get_f64(o)
     }
-    pub fn set_f64(&self, o: &[usize], v: f64) {
-        self.var.borrow_mut().set_f64(o, v);
+    pub fn set_f64(&self, o: &[usize], v: f64) -> Result<(), AutoDiffError> {
+        self.var.borrow_mut().set_f64(o, v)
     }
 
     pub fn set(&self, o: &Var) {
@@ -121,11 +153,14 @@ impl Var {
     delegate_new_op!(fill_f64, size: &[usize], fill_value: f64);
     delegate_new_op!(zeros, dim: &[usize]);
     delegate_new_op!(ones, dim: &[usize]);
+    delegate_new_op!(twos, dim: &[usize]);
     //delegate_new_inner_op!(arange, end: usize);
     //delegate_new_inner_op!(range, start: f32, end: f32, step: Option<f32>);
     //delegate_new_inner_op!(linspace, start: f32, end: f32, steps: usize);
     //delegate_new_inner_op!(logspace, start: f32, end: f32, steps: usize, base: f32);
-    delegate_new_op!(eye, n: usize, m: usize);
+    delegate_new_op!(
+        /// Identity matrix
+        eye, n: usize, m: usize);
     delegate_new_op!(empty, dim: &[usize]);
     
     /// Fill row by row.
@@ -184,12 +219,64 @@ impl Var {
         Self::uniform_f64(rng, dim, from, to)
     }
 
-
-    var_2_to_1!(add);
-    var_2_to_1!(sub);
-    var_2_to_1!(mul);
-    var_2_to_1!(div);
-    var_2_to_1!(matmul);
+    pub fn _add(&self, other: &Var) -> Var {
+        Var {
+            var: Rc::new(RefCell::new(self.var.borrow().add(&other.var).expect("never fail.")))
+        }
+    }
+    pub fn _sub(&self, other: &Var) -> Var {
+        Var {
+            var: Rc::new(RefCell::new(self.var.borrow().sub(&other.var).expect("never fail.")))
+        }
+    }
+    pub fn _mul(&self, other: &Var) -> Var {
+        Var {
+            var: Rc::new(RefCell::new(self.var.borrow().mul(&other.var).expect("never fail.")))
+        }
+    }
+    pub fn _div(&self, other: &Var) -> Var {
+        Var {
+            var: Rc::new(RefCell::new(self.var.borrow().div(&other.var).expect("never fail.")))
+        }
+    }
+    
+    var_2_to_1!(
+        /// Matrix/inner/dot product
+        /// 
+        /// # use auto_diff::{Var, var_f64, AutoDiffError};
+        /// # extern crate openblas_src;
+        /// # fn test_matmul() -> Result<(), AutoDiffError> {
+        /// let v1 = var_f64!([[1., 2., 3.],
+        ///                    [4., 5., 6.]]);
+        /// let v2 = var_f64!([[11., 12., 13.],
+        ///                    [14., 15., 16.],
+        ///                    [17., 18., 19.]]);
+        /// let v3 = v1.matmul(&v2)?;
+        /// let em = var_f64!([[90.0, 96.0, 102.0],
+        ///                    [216.0, 231.0, 246.0]]);
+        /// assert_eq!(v3, em);
+        /// #   Ok(())
+        /// # }
+        /// # test_matmul();
+        /// 
+        matmul);
+    var_2_to_1!(
+        /// Outer product
+        /// ```
+        /// # use auto_diff::{Var, var_f64, AutoDiffError};
+        /// # fn test_outer() -> Result<(), AutoDiffError> {
+        /// let v1 = Var::new_f64(&[1., 2., 3.], &[3]);
+        /// let v2 = Var::new_f64(&[4., 5., 6.], &[3]);
+        /// let v3 = v1.outer(&v2)?;
+        /// let em = var_f64!([[4.,   5.,  6.],
+        ///                    [8.,  10., 12.],
+        ///                    [12., 15., 18.]]);
+        /// assert_eq!(v3, em);
+        /// #   Ok(())
+        /// # }
+        /// # test_outer();
+        /// ```
+        outer);
 
     // nonlinear
     pub fn elu(&self, alpha: Var) -> Result<Var, AutoDiffError> {
@@ -225,6 +312,11 @@ impl Var {
     var_1_to_1!(log1pexp);
     var_1_to_1!(log2);
     var_1_to_1!(neg);
+    pub fn _neg(&self) -> Var {
+        Var {
+            var: Rc::new(RefCell::new(self.var.borrow().neg().expect("never fail.")))
+        }
+    }
     var_1_to_1!(reciprocal);
     var_1_to_1!(round);
     var_1_to_1!(rsqrt);
@@ -250,7 +342,31 @@ impl Var {
     var_2_to_1!(ne);
 
     // index and slicing
-    var_more_to_1_with_para!(cat, dim: usize);
+    var_more_to_1_with_para!(
+        /// Concatenates the given sequence of seq tensors
+        /// in the given dimension.
+        /// The input tensor should all have the same size except
+        /// on the given dimension.
+        /// The output tensor will have all the same size as the input
+        /// except the given dimension, which will be the sum of
+        /// the inputs on the given dimension.
+        /// Apply cat on [tensor(5, 3, 2), tensor(5, 7, 2), ]
+        /// will get a tensor(5, 10, 2).
+        ///
+        /// 
+        /// # use auto_diff::{Var, var_f64, AutoDiffError};
+        /// # extern crate openblas_src;
+        /// # fn test_cat() -> Result<(), AutoDiffError> {
+        /// let m1 = Var::empty(&[3, 1]);
+        /// let m2 = Var::empty(&[3, 1]);
+        /// let m3 = Var::empty(&[3, 1]);
+        /// let m4 = m1.cat(&[m2, m3], 1)?;
+        /// assert_eq!(m4.size(), [3, 3]);
+        /// #   Ok(())
+        /// # }
+        /// # test_cat();
+        /// 
+        cat, dim: usize);
     pub fn chunk(&self, chunks: usize, dim: usize)
                  -> Result<Vec<Var>, AutoDiffError> {
         let mut result = self.var.borrow().chunk(chunks, dim)?;
@@ -349,12 +465,32 @@ impl Var {
             var: Rc::new(RefCell::new(result)),
         })
     }
-    var_more_to_1_with_para!(stack, dim: usize);
+    var_more_to_1_with_para!(
+        /// Stack tensor with the same size along a new dimension
+        /// specified by dim.
+        /// The difference from cat is that cat don't create new dimension.
+        ///
+        /// ```
+        /// # use auto_diff::{Var, var_f64, AutoDiffError};
+        /// # fn test_stack() -> Result<(), AutoDiffError> {
+        /// let m1 = var_f64!([[1., 2., ],
+        ///                [3., 4., ]]);
+        /// let m2 = var_f64!([[5., 6., ],
+        ///                [7., 8., ]]);
+        /// let m3 = m1.stack(&[m2], 1)?;
+        /// #   let em = Var::new_f64(&[1.0, 2.0, 5.0, 6.0, 3.0, 4.0, 7.0, 8.0], &[2, 2, 2]);
+        /// #   assert_eq!(m3, em);
+        /// #   Ok(())
+        /// # }
+        /// # test_stack();
+        /// ```
+        stack, dim: usize);
 
     // linalg
     var_1_to_1!(det);
     var_1_to_1!(inv);
     var_1_to_1!(normalize_unit);
+    var_1_to_1!(tr);
 
     // reduction
     var_1_to_1_with_para!(argmax, dim: Option<&[usize]>, keepdim: bool);
@@ -369,7 +505,43 @@ impl Var {
     var_1_to_1_with_para!(min, dim: Option<&[usize]>, keepdim: bool);
 
     // images
-    var_1_to_1_with_para!(get_patch, range: &[(usize, usize)], step: Option<&[usize]>);
+    var_1_to_1_with_para!(
+        /// Get a portion of the tensor and return it.
+        ///
+        /// ```
+        /// # use auto_diff::{Var, var_f64, AutoDiffError};
+        /// # fn test_get_patch() -> Result<(), AutoDiffError> {
+        /// let m1 = var_f64!([[1., 2., 3.],
+        ///                    [4., 5., 6.],
+        ///                    [7., 8., 9.]]);
+        /// let m2 = var_f64!([[4., 5.],
+        ///                    [7., 8.]]);
+        /// assert_eq!(m1.get_patch(&[(1, 3), (0, 2)], None)?, m2);
+        /// #   Ok(())
+        /// # }
+        /// # test_get_patch();
+        /// ```
+        get_patch, range: &[(usize, usize)], step: Option<&[usize]>);
+    var_2_to_1_with_para!(
+        /// Set a portion of the tensor.
+        ///
+        /// ```
+        /// # use auto_diff::{Var, var_f64, AutoDiffError};
+        /// # fn test_set_patch() -> Result<(), AutoDiffError> {
+        /// let m1 = var_f64!([[1., 2., 3.],
+        ///                    [4., 5., 6.],
+        ///                    [7., 8., 9.]]);
+        /// let m2 = var_f64!([[10., 11.],
+        ///                    [12., 13.]]);
+        /// let m3 = var_f64!([[1.,   2., 3.],
+        ///                    [10., 11., 6.],
+        ///                    [12., 13., 9.]]);
+        /// assert_eq!(m1.set_patch(&m2, &[(1, 3), (0, 2)], None)?, m3);
+        /// #   Ok(())
+        /// # }
+        /// # test_set_patch();
+        /// ```
+        set_patch, range: &[(usize, usize)], step: Option<&[usize]>);
     var_1_to_1_with_para!(view, new_shape: &[usize]);
 
 
@@ -395,6 +567,7 @@ impl Var {
         })
     }
 
+    /// Apply back propagation to get numerical gradient.
     pub fn bp(&self) -> Result<(), AutoDiffError> {
         self.var.borrow().bp()?;
 
@@ -405,6 +578,7 @@ impl Var {
         self.var.borrow().step(opt)
     }
 
+    /// Run the computation graph again.
     pub fn rerun(&self) -> Result<(), AutoDiffError> {
         self.var.borrow().rerun()
     }
@@ -419,11 +593,13 @@ impl Var {
         Ok(ret)
     }
 
+    /// For debug.
     pub fn dump_net(&self) -> Rc<RefCell<Net>> {
         self.var.borrow().dump_net()
     }
 }
 
+// Test for equal
 impl PartialEq for Var {
     fn eq(&self, other: &Self) -> bool {
         self.var.borrow().val().eq(&other.var.borrow().val())
@@ -454,20 +630,142 @@ impl Clone for Var {
     }
 }
 
+// Operator overloading
+impl Add for Var {
+    type Output = Self;
+
+    fn add(self, other: Self) -> Self {
+        self._add(&other)
+    }
+}
+
+impl Sub for Var {
+    type Output = Self;
+
+    fn sub(self, other: Self) -> Self {
+        self._sub(&other)
+    }
+}
+
+impl Mul for Var {
+    type Output = Self;
+
+    fn mul(self, other: Self) -> Self {
+        self._mul(&other)
+    }
+}
+
+impl Div for Var {
+    type Output = Self;
+
+    fn div(self, other: Self) -> Self {
+        self._div(&other)
+    }
+}
+
+impl Neg for Var {
+    type Output = Self;
+
+    fn neg(self) -> Self {
+        self._neg()
+    }
+}
+
+impl TryFrom<Var> for f32 {
+    type Error = AutoDiffError;
+
+    fn try_from(value: Var) -> Result<Self, Self::Error> {
+	let index = vec![0; value.size().len()];
+	value.get_f32(&index)
+    }
+}
+
+impl TryFrom<Var> for f64 {
+    type Error = AutoDiffError;
+
+    fn try_from(value: Var) -> Result<Self, Self::Error> {
+	let index = vec![0; value.size().len()];
+	value.get_f64(&index)
+    }
+}
 
 
 
+#[macro_export]
+macro_rules! var_f64 {
+    ($a:expr) => {{
+        trait Expand {
+            fn expand(&self) -> Var;
+        }
+        impl Expand for [f64] {
+            fn expand(&self) -> Var {
+                Var::new_f64(&self, &[self.len(), 1])
+            }
+        }
+        impl Expand for [[f64; 1]] {
+            fn expand(&self) -> Var {
+                let mut data = vec![];
+                let m = self.len();
+                let mut n = 0;
+                for i in self {
+                    n = i.len();
+                    data.append(&mut i.to_vec());
+                }
+                Var::new_f64(&data, &[m, n])
+            }
+        }
+        impl Expand for [[f64; 2]] {
+            fn expand(&self) -> Var {
+                let mut data = vec![];
+                let m = self.len();
+                let mut n = 0;
+                for i in self {
+                    n = i.len();
+                    data.append(&mut i.to_vec());
+                }
+                Var::new_f64(&data, &[m, n])
+            }
+        }
+        impl Expand for [[f64; 3]] {
+            fn expand(&self) -> Var {
+                let mut data = vec![];
+                let m = self.len();
+                let mut n = 0;
+                for i in self {
+                    n = i.len();
+                    data.append(&mut i.to_vec());
+                }
+                Var::new_f64(&data, &[m, n])
+            }
+        }
+        impl Expand for [[f64; 4]] {
+            fn expand(&self) -> Var {
+                let mut data = vec![];
+                let m = self.len();
+                let mut n = 0;
+                for i in self {
+                    n = i.len();
+                    data.append(&mut i.to_vec());
+                }
+                Var::new_f64(&data, &[m, n])
+            }
+        }
+        $a.expand()
+    }}
+}
+
 
 #[cfg(test)]
 mod tests {
     use super::*;
     use crate::op::OpCall;
+    extern crate openblas_src;
 
     #[test]
     fn mul() {
         let a = Var::new(&[2., 3., 4., 5.], &[2, 2]);
         let b = Var::new(&[1., 2., 3., 4.], &[2, 2]);
-        let c = a.mul(&b).unwrap();
+        let c = a.ref_copy() * b.ref_copy();
         assert_eq!(c, Var::new(&[2., 6., 12., 20.], &[2, 2]));
         c.bp().unwrap();
         assert_eq!(a.grad().unwrap(), Var::new(&[1., 2., 3., 4.], &[2, 2]));
@@ -478,8 +776,8 @@ mod tests {
     fn test_mul_repeat_vars() {
         let a = Var::new(&[2., 3., 4., 5.], &[2, 2]);
         let b = Var::new(&[1., 2., 3., 4.], &[2, 2]);
-        let c = a.mul(&b).unwrap();
-        let d = c.mul(&b).unwrap(); // repeat vars
+        let c = a * b.ref_copy();
+        let d = c * b; // repeat vars
         assert_eq!(d, Var::new(&[2., 12., 36., 80.], &[2, 2]));
     }
 
@@ -489,7 +787,7 @@ mod tests {
         let b = Var::new(&[1., 2., 3., 4.], &[2, 2]);
     
         fn my_mul(a: &Var, b: &Var) -> Var {
-            a.mul(b).unwrap()
+            a.ref_copy() * b.ref_copy()
         }
         let c = my_mul(&a, &b);
         assert_eq!(c, Var::new(&[2., 6., 12., 20.], &[2, 2]));
@@ -515,4 +813,22 @@ mod tests {
         assert_eq!(output, Var::new(&[8.0, 11.0, 14.0, 17.0, 20.0, 8.0, 11.0, 14.0, 17.0, 20.0, 8.0, 11.0, 14.0, 17.0, 20.0],
                                     &vec![3, 5]));
     }
+
+    #[test]
+    fn test_macro_tensor() {
+        let a = var_f64!([1., 2., 3.,]);
+        assert_eq!(a.size(), [3, 1]);
+        let a1 = a.squeeze(None).unwrap();
+        println!("{:?}", a1.size());
+        let a = var_f64!([[1., 2.,],
+                          [4., 5.,],
+                          [4., 5.,]]);
+        assert_eq!(a.size(), [3, 2]);
+        let a = var_f64!([[1., 2., 3.,],
+                          [4., 5., 6.,]]);
+        assert_eq!(a.size(), [2, 3]);
+        let a = var_f64!([[1., 2., 3., 7.],
+                          [4., 5., 6., 8.]]);
+        assert_eq!(a.size(), [2, 4]);
+    }
 }
