@@ -7,79 +7,31 @@ use ::rand::prelude::StdRng;
 use tensor_rs::tensor::{Tensor};
 use crate::compute_graph::{Net};
 use crate::collection::generational_index::{GenKey};
-use crate::op::{Op, OpTrait,
+use crate::op::{Op,
                 View,
-                Add, Sub, Mul, Div, Matmul,
+                Add, Sub, Mul, Div, Matmul, Outer,
                 ELU, ReLU, Sigmoid,
                 MSELoss, BCEWithLogitsLoss, CrossEntropyLoss,
                 Abs, Acos, Asin, Atan, Ceil, Cos, Cosh, Exp, Expm1, Floor, Frac, Log, Log10, Log1p, Log1pexp, Log2, Neg, Reciprocal, Round, Rsqrt, Sign, Sin, Sinh, Sqrt, Tan, Tanh, Trunc,
                 MaxPair, MinPair, ArgSort, EqElem, Equal, Ge, Gt, Le, Lt, Ne,
                 Cat, Chunk, Gather, IndexSelect, IndexExclude, Reshape, Split, Squeeze, Stack, T, Take, Permute, Unsqueeze, ConditionalSelect, Repeat,
-                Det, Inv, NormalizeUnit,
+                Det, Inv, NormalizeUnit, Tr,
                 Argmax, Argmin, Logsumexp, Mean, Prod, Std, Sum, Variance, Max, Min,
-                GetPatch,
+                GetPatch, SetPatch,
 };
 use crate::err::AutoDiffError;
 use crate::optim::Optimizer;
 
 
-//macro_rules! var_inner_1_to_1_with_args {
-//    ($a:ident, $b:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
-//        pub fn $a(&self, $( $arg_name : $ArgTy ),*) -> Result<VarInner, AutoDiffError> {
-//            if self.need_grad {
-//                let ret = VarInner::new_net_tensor(self.net.clone(), Tensor::new());
-//                let op = $b::new($( $arg_name ),*);
-//                op.apply(&[self.net.borrow().get_tensor(self.id)?.ref_copy()],
-//                         &[self.net.borrow().get_tensor(ret.id)?.ref_copy()]);
-//                let op = Op::new(Rc::new(RefCell::new(Box::new(op))));
-//                let opid = self.net.borrow_mut().add_op(op);
-//                
-//                self.net.borrow_mut().connect(&[self.id],
-//                                              opid, &[ret.id]);
-//                
-//                Ok(ret)
-//            } else {
-//                let ret = VarInner::new_net_tensor(Rc::new(RefCell::new(Net::new())), Tensor::new());
-//                let op = $b::new($( $arg_name ),*);
-//                op.apply(&[self.net.borrow().get_tensor(self.id)?.ref_copy()],
-//                         &[ret.net.borrow().get_tensor(ret.id)?.ref_copy()]);
-//                Ok(ret)
-//            }
-//        }
-//    }
-//}
-
 /// For elementwise ops
 /// var_inner_1_to_1!(abs, Abs);
 macro_rules! var_inner_1_to_1 {
     ($a:ident, $b:ident) => {
         pub fn $a(&self) -> Result<VarInner, AutoDiffError> {
-            if self.need_grad {
-                
-                let ret = VarInner::new_net_tensor(self.net.clone(),
-                                                   self.need_grad,
-                                                   Tensor::new());
-                
-                let op = $b::new();
-                op.apply(&[self.net.borrow().get_tensor(self.id)?.ref_copy()],
-                         &[self.net.borrow().get_tensor(ret.id)?.ref_copy()]);
-                let op = Op::new(Rc::new(RefCell::new(Box::new(op))));
-                let opid = self.net.borrow_mut().add_op(op);
-                
-                self.net.borrow_mut().connect(&[self.id],
-                                              opid, &[ret.id]);
-                
-                Ok(ret)
-            } else {
-                let ret = VarInner::new_net_tensor(Rc::new(RefCell::new(Net::new())),
-                                                   self.need_grad,
-                                                   Tensor::new());
-                let op = $b::new();
-                op.apply(&[self.net.borrow().get_tensor(self.id)?.ref_copy()],
-                         &[ret.net.borrow().get_tensor(ret.id)?.ref_copy()]);
-                Ok(ret)
-            }
-            
+            let new_one = $b::new();
+            let op = Op::new(Rc::new(RefCell::new(Box::new(new_one))));
+            let mut result = self.called_with(op, &[])?;
+            Ok(result.remove(0))            
         }
     }
 }
@@ -87,42 +39,12 @@ macro_rules! var_inner_1_to_1 {
 
 macro_rules! var_inner_2_to_1 {
     ($a:ident, $b:ident) => {
-        pub fn $a(&self, other: &mut VarInner) -> Result<VarInner, AutoDiffError> {
-            if self.need_grad {
-                if !Rc::ptr_eq(&self.net, &other.net) {
-                    let other_key = self.net.borrow_mut().append(
-                        &other.net.borrow(), &[other.id])?[0];
-                
-                    other.net = self.net.clone();
-                    other.id = other_key;
-                }
-                
-                let ret = VarInner::new_net_tensor(self.net.clone(),
-                                                   self.need_grad,
-                                                   Tensor::new());
-                
-                let op = $b::new();
-                op.apply(&[self.net.borrow().get_tensor(self.id)?.ref_copy(),
-                           self.net.borrow().get_tensor(other.id)?.ref_copy()],
-                         &[self.net.borrow().get_tensor(ret.id)?.ref_copy()]);
-                let op = Op::new(Rc::new(RefCell::new(Box::new(op))));
-                let opid = self.net.borrow_mut().add_op(op);
-                
-                self.net.borrow_mut().connect(&[self.id, other.id],
-                                              opid, &[ret.id]);
-                
-                Ok(ret)
-            } else {
-                let ret = VarInner::new_net_tensor(Rc::new(RefCell::new(Net::new())),
-                                                   self.need_grad,
-                                                   Tensor::new());
-                let op = $b::new();
-                op.apply(&[self.net.borrow().get_tensor(self.id)?.ref_copy(),
-                           other.net.borrow().get_tensor(other.id)?.ref_copy()],
-                         &[ret.net.borrow().get_tensor(ret.id)?.ref_copy()]);
-                Ok(ret)
-            }
-            
+        pub fn $a(&self, other: &Rc<RefCell<VarInner>>) -> Result<VarInner, AutoDiffError> {
+            let new_one = $b::new();
+            let op = Op::new(Rc::new(RefCell::new(Box::new(new_one))));
+            let o_input = vec![other.clone()];
+            let mut result = self.called_with(op, &o_input)?;
+            Ok(result.remove(0))            
         }
     }
 }
@@ -151,6 +73,19 @@ macro_rules! var_inner_1_to_1_with_para {
     }
 }
 
+macro_rules! var_inner_2_to_1_with_para {
+    ($a:ident, $b:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
+        pub fn $a(&self, other: &Rc<RefCell<VarInner>>,
+                  $( $arg_name : $ArgTy ),*)
+                  -> Result<VarInner, AutoDiffError> {
+            let new_one = $b::new($( $arg_name ),*);
+            let op = Op::new(Rc::new(RefCell::new(Box::new(new_one))));
+            let mut result = self.called_with(op, &[other.clone()])?;
+            Ok(result.remove(0))            
+        }
+    }
+}
+
 
 
 // Macro for creation associated function.
@@ -259,17 +194,30 @@ impl VarInner {
     pub fn numel(&self) -> usize {
         self.net.borrow().get_tensor(self.id).expect("").numel()
     }
-    pub fn get_f32(&self, o: &[usize]) -> f32 {
-        self.net.borrow().get_tensor(self.id).expect("").get_f32(o)
+    fn check_index(v: &VarInner, o: &[usize]) -> Result<(), AutoDiffError> {
+	if v.size().len() != o.len() {
+	    return Err(AutoDiffError::new(
+		&format!("Index for get() should have the same len. t: {:?}, index: {:?}",
+			 v.size(), o.len())));
+	} else {
+	    Ok(())
+	}
     }
-    pub fn set_f32(&mut self, o: &[usize], v: f32) {
-        self.net.borrow().get_tensor(self.id).expect("").set_f32(o, v);
+    pub fn get_f32(&self, o: &[usize]) -> Result<f32, AutoDiffError> {
+	Self::check_index(&self, o)?;
+        Ok(self.net.borrow().get_tensor(self.id)?.get_f32(o))
     }
-    pub fn get_f64(&self, o: &[usize]) -> f64 {
-        self.net.borrow().get_tensor(self.id).expect("").get_f64(o)
+    pub fn set_f32(&mut self, o: &[usize], v: f32) -> Result<(), AutoDiffError> {
+	Self::check_index(&self, o)?;
+        Ok(self.net.borrow().get_tensor(self.id)?.set_f32(o, v))
     }
-    pub fn set_f64(&mut self, o: &[usize], v: f64) {
-        self.net.borrow().get_tensor(self.id).expect("").set_f64(o, v);
+    pub fn get_f64(&self, o: &[usize]) -> Result<f64, AutoDiffError> {
+	Self::check_index(&self, o)?;
+        Ok(self.net.borrow().get_tensor(self.id)?.get_f64(o))
+    }
+    pub fn set_f64(&mut self, o: &[usize], v: f64) -> Result<(), AutoDiffError>{
+	Self::check_index(&self, o)?;
+        Ok(self.net.borrow().get_tensor(self.id)?.set_f64(o, v))
     }
 
     pub fn fill(size: &[usize], fill_value: Rc<RefCell<VarInner>>) -> VarInner {
@@ -304,6 +252,7 @@ impl VarInner {
     }
     delegate_new_inner_op!(zeros, dim: &[usize]);
     delegate_new_inner_op!(ones, dim: &[usize]);
+    delegate_new_inner_op!(twos, dim: &[usize]);
     //delegate_new_inner_op!(arange, end: usize);
     //delegate_new_inner_op!(range, start: f32, end: f32, step: Option<f32>);
     //delegate_new_inner_op!(linspace, start: f32, end: f32, steps: usize);
@@ -399,16 +348,35 @@ impl VarInner {
                               others: &[Rc<RefCell<VarInner>>])
                               -> Result<Vec<VarInner>, AutoDiffError> {
         if self.need_grad {
-            // TODO there may the same net among others.
+            let mut other_var_by_networks: Vec<Vec<Rc<RefCell<VarInner>>>> = vec![];
             for item in others.iter().cloned() {
                 if !Rc::ptr_eq(&self.net, &item.borrow().net) {
-                    let other_key = self.net.borrow_mut().append(
-                        &item.borrow().net.borrow(), &[item.borrow().id])?[0];
-            
-                    item.borrow_mut().net = self.net.clone();
-                    item.borrow_mut().id = other_key;
+                    let mut existing_net = false;
+                    for set in &mut other_var_by_networks {
+                        if Rc::ptr_eq(&item.borrow().net, &set[0].borrow().net) {
+                            set.push(item.clone());
+                            existing_net = true;
+                            break;
+                        }
+                    }
+                    if ! existing_net {
+                        other_var_by_networks.push(vec![item.clone()]);
+                    }
                 }
             }
+            for set in other_var_by_networks {
+                let mut old_ids = vec![];
+                for item in &set {
+                    old_ids.push(item.borrow().id.clone());
+                }
+                let other_key = self.net.borrow_mut().append(
+                    &set[0].borrow().net.borrow(), &old_ids)?;
+                for (index, item) in set.iter().enumerate() {
+                    item.borrow_mut().net = self.net.clone();
+                    item.borrow_mut().id = other_key[index];
+                }
+
+            }
             
             let mut input_id = vec![self.id];
             let mut inputs = vec![self.net.borrow().get_tensor(self.id)?];
@@ -428,7 +396,7 @@ impl VarInner {
                 outputs.push(self.net.borrow().get_tensor(new_output.id)?);
                 ret.push(new_output);
             }
-            
+
             op.apply(&inputs, &outputs);
             let opid = self.net.borrow_mut().add_op(op);
             
@@ -465,6 +433,7 @@ impl VarInner {
     var_inner_2_to_1!(mul, Mul);
     var_inner_2_to_1!(div, Div);
     var_inner_2_to_1!(matmul, Matmul);
+    var_inner_2_to_1!(outer, Outer);
 
     // nonlinear
     pub fn elu(&self, alpha: VarInner) -> Result<VarInner, AutoDiffError> {
@@ -610,6 +579,7 @@ impl VarInner {
     var_inner_1_to_1!(det, Det);
     var_inner_1_to_1!(inv, Inv);
     var_inner_1_to_1!(normalize_unit, NormalizeUnit);
+    var_inner_1_to_1!(tr, Tr);
 
     // reduction
     var_inner_1_to_1_with_para!(argmax, Argmax, dim: Option<&[usize]>, keepdim: bool);
@@ -625,6 +595,7 @@ impl VarInner {
 
     // images
     var_inner_1_to_1_with_para!(get_patch, GetPatch, range: &[(usize, usize)], step: Option<&[usize]>);
+    var_inner_2_to_1_with_para!(set_patch, SetPatch, range: &[(usize, usize)], step: Option<&[usize]>);
     var_inner_1_to_1_with_para!(view, View, new_shape: &[usize]);
 
     pub fn dump_net(&self) -> Rc<RefCell<Net>> {
